/*
 *  memory_solaris.cpp
 *
 *  Torsten Kasch <tk@Genetik.Uni-Bielefeld.DE>
 */

#include <kstat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <inttypes.h>
#include <sys/sysinfo.h>
#include <sys/stat.h>
#include <vm/anon.h>

#include "memory.h"

#ifdef PAGETOK
#undef PAGETOK
#endif
#define PAGETOK(a) (((t_memsize) sysconf(_SC_PAGESIZE)) * (t_memsize) a)

extern "C" {
static unsigned long interval = 0UL;
time_t ot = (time_t) 0;
time_t nt = (time_t) 0;
static vminfo_t ovminfo;
static vminfo_t nvminfo;
static unsigned long pagesize = 0UL;
static uint64_t physmem = 0ULL;
static const unsigned long long mystery = 1048576ULL;
static bool skip = true;
static const t_memsize MB = t_memsize(1024ULL * 1024ULL);

inline t_memsize delta (t_memsize n, t_memsize o)
{
    return ((t_memsize) (n - o) * pagesize / mystery / interval);
}

static void getVMinfo (t_memsize* freemem,
		       t_memsize* reserved,
		       t_memsize* allocated,
		       t_memsize* available,
		       t_memsize* freeswap)
{
    *freemem = delta (nvminfo.freemem, ovminfo.freemem);
    *reserved = delta (nvminfo.swap_resv, ovminfo.swap_resv);
    *allocated = delta (nvminfo.swap_alloc, ovminfo.swap_alloc);
    *available = delta (nvminfo.swap_avail, ovminfo.swap_avail);
    *freeswap = delta (nvminfo.swap_free, ovminfo.swap_free);
}

static unsigned long updateTime()
{
    ot = nt;
    nt = time(0);

    if ((time_t) 0 == ot)
	    return 0UL;

    interval = (unsigned long) nt - ot;
    return interval;
}
} // extern "C"

static void setDefaultValues()
{
    Memory_Info[FREE_MEM] = t_memsize(0);
    Memory_Info[SWAP_MEM] = t_memsize(0);
    Memory_Info[FREESWAP_MEM] = t_memsize(0);
    Memory_Info[INACTIVE_MEM] = t_memsize(0);
    Memory_Info[TOTAL_MEM] = t_memsize(0);
    Memory_Info[CACHED_MEM] = t_memsize(0);
    Memory_Info[ACTIVE_MEM] = t_memsize(0);
}

void KMemoryWidget::update()
{
	kstat_ctl_t* kctl = 0L;
	kstat_t* ksp = 0L;
	kstat_t* kmp = 0L;
	kstat_named_t* knp = 0L;

	t_memsize freemem = t_memsize(0);
	t_memsize reserved = t_memsize(0);
	t_memsize allocated = t_memsize(0);
	t_memsize available = t_memsize(0);
	t_memsize freeswap = t_memsize(0);
	t_memsize totalmem = t_memsize(0);
	t_memsize pmem = t_memsize(0);
	t_memsize availrmem = t_memsize(0);
	t_memsize ramcachemem = t_memsize(0);

	pagesize = sysconf (_SC_PAGESIZE);
  physmem = PAGETOK(sysconf(_SC_PHYS_PAGES)) / (1024ULL * 1024ULL);
	pmem = physmem;

	if ((kctl = kstat_open()) == NULL)
	    return;

	if (0UL == updateTime())
	{
    kstat_close (kctl);
	  return;
	}

	ksp = kstat_lookup(kctl, "unix", 0, "vminfo");
	if (NULL == ksp)
  {
    kstat_close (kctl);
	  return;
  }
	
	if ((kid_t) -1 == kstat_read (kctl, ksp, &nvminfo))
  {
    kstat_close (kctl);
	  return;
  }

	getVMinfo (&freemem, &reserved, &allocated, &available, &freeswap);
	ovminfo = nvminfo;
	if (skip)
	{
	    setDefaultValues();
	    skip = false;
	    kstat_close (kctl);
	    return;
	}
	Memory_Info[FREE_MEM] = freemem * MB;
	Memory_Info[SWAP_MEM] = freeswap * MB;
	Memory_Info[FREESWAP_MEM] = (available - reserved) * MB;

	if ((kmp = kstat_lookup (kctl, "unix", 0, "system_pages")) == (kstat_t *) NULL)
  {
    kstat_close (kctl);
    return;
  }

  while (kstat_chain_update(kctl) != 0);

	if (kstat_read (kctl, kmp, NULL) != -1)
	{
	    knp = (kstat_named_t *) kstat_data_lookup (kmp, "physmem");
	    if (knp)
	    {
        totalmem = physmem;
		    ramcachemem = totalmem;
		    Memory_Info[TOTAL_MEM] = totalmem;
		    Memory_Info[ACTIVE_MEM] = (totalmem - freemem);
		    Memory_Info[ACTIVE_MEM] *= MB;
		    Memory_Info[INACTIVE_MEM] = (freemem + (reserved - allocated));
		    Memory_Info[INACTIVE_MEM] *= MB;
	    }
	}
  kstat_read (kctl, kmp, NULL);
	knp = (kstat_named_t *) kstat_data_lookup (kmp, "availrmem");
	if (knp)
	{
	    availrmem = PAGETOK(knp->value.ui32) / mystery;
      ramcachemem -= availrmem;
	    Memory_Info[CACHED_MEM] = ramcachemem * MB;
	    Memory_Info[SHARED_MEM] = ((reserved - allocated) + ramcachemem) * MB;
      Memory_Info[TOTAL_MEM] *= MB;
	}
	kstat_close (kctl);
}

// vim: ts=2 sw=2 et

