/*
    KSysGuard, the KDE System Guard

    Copyright (c) 2000-2001 Chris Schlaeger <cs@kde.org>
    Solaris 8 9 and 10 support by Stefan Teleman <stefan.teleman@gmail.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of version 2 of the GNU General Public
    License as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "config.h"

#include <kstat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <sys/sysinfo.h>
#include <errno.h>

#include "Command.h"
#include "ccont.h"
#include "CpuInfo.h"
#include "ksysguardd.h"

typedef uint32_t CPUInstance;
typedef uint64_t CPUId;

typedef struct CPUInfo
{
  CPUId id;
  CPUInstance instance;
  uint32_t clockMHz;
  uint32_t online;
  char name[32];
  char cpuType[32];
  char fpuType[32];
  char implementation[64];
  char state[16];
} CPUInfo;

typedef struct CPUIndex
{
  uint32_t id;
  uint32_t index;
} CPUIndex;

extern int errno;

static int Dirty = 0;
static int CpuDevCnt = 0;
static struct SensorModul* CpuDevSM;

static unsigned long interval = 0UL;
static time_t ot = (time_t) 0;
static time_t nt = (time_t) 0;

static int loop = 1;
static uint32_t ncpuread = 0;
static long ncpuonln = 0L; // sysconf(_SC_NPROCESSORS_ONLN);
static long ncpuconf = 0L; // sysconf(_SC_NPROCESSORS_CONF);
static long ncpumax = 0L; // sysconf(_SC_NPROCESSORS_MAX);
static cpu_stat_t* cpuOldStatArray = 0L;
static cpu_stat_t* cpuNewStatArray = 0L;
static CPUInfo* cpuInfoArray = 0L;
static CPUIndex* cpuIndexArray = 0L;

static double* cpuIdle = 0L;
static double* cpuUser = 0L;
static double* cpuKernel = 0L;
static double* cpuWait = 0L;
static double* waitIO = 0L;
static double* waitBlockIO = 0L;
static double* waitSwap = 0L;
static double* waitPIO = 0L;

static double
ddelta32 (uint32_t n, uint32_t o)
{
  return floor(((double) ((n - o) / interval)));
}

static int
isOnline (const CPUInfo* cpuInfo)
{
  return ((cpuInfo) ? cpuInfo->online : 0);
}

static int32_t
getCpuIndex (int32_t id)
{
  CPUId cpuId;
  uint32_t i;

  if (0L == cpuIndexArray)
    return -1;

  cpuId = (CPUId) id;
  for (i = 0; i < ncpuread; i++)
  {
    if (cpuIndexArray[i].id == id)
      return cpuIndexArray[i].index;
  }
  return -1;
}

static int32_t
getCpuId (const char* cmd)
{
  char cpuId[32];
  uint32_t i = 0;

  if (0L == cmd)
    return -1;

  const char* slash = strchr (cmd, '/');
  if (0L == slash)
    return -1;

  const char* ptr = cmd;
  ptr += 3 * sizeof(char);
  (void) memset (cpuId, '\0', (size_t) 32);

  while (ptr != slash)
    cpuId[i++] = *ptr++;

  return (atoi(cpuId));
}

static unsigned long
updateTime (void)
{
  ot = nt;
  nt = time(0);

  if ((time_t) 0 == ot)
    return 0UL;

  interval = (unsigned long) nt - ot;
  return interval;
}

static void
getCPUInfo (kstat_t* kmp, CPUInfo* cpuInfo)
{
  kstat_named_t* kcpuinfop = 0L;

  if ((0 == kmp) || (0 == cpuInfo))
    return;

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "chip_id");
  if (kcpuinfop)
    cpuInfo->id = kcpuinfop->value.ui64;

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "clock_MHz");
  if (kcpuinfop)
    cpuInfo->clockMHz = kcpuinfop->value.ui32;

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "cpu_type");
  if (kcpuinfop)
    (void) strcpy (cpuInfo->cpuType, kcpuinfop->value.c);

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "fpu_type");
  if (kcpuinfop)
    (void) strcpy (cpuInfo->fpuType, kcpuinfop->value.c);

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "implementation");
  if (kcpuinfop)
  {
#if defined (SOLARIS8)
    (void) strcpy (cpuInfo->implementation, kcpuinfop->value.c);
#else
    (void) strcpy (cpuInfo->implementation, kcpuinfop->value.string.addr.ptr);
#endif
  }

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "state");
  if (kcpuinfop)
  {
    (void) strcpy (cpuInfo->state, kcpuinfop->value.c);
    cpuInfo->online = (0 == strcmp(kcpuinfop->value.c, "on-line"));
  }
}

static void
setDefaultCPUValues (void)
{
  uint32_t i;

  for (i = 0; i < (uint32_t) ncpuconf; i++)
  {
    (void) memset ((void *) &(cpuInfoArray[i]), '\0', sizeof (struct CPUInfo));
    (void) memset ((void *) &(cpuIndexArray[i]), '\0', sizeof (struct CPUIndex));
    (void) memset ((void *) &(cpuNewStatArray[i]), '\0', sizeof (cpu_stat_t));
    (void) memset ((void *) &(cpuOldStatArray[i]), '\0', sizeof (cpu_stat_t));
    cpuIdle[i] = 0.0;
    cpuUser[i] = 0.0;
    cpuKernel[i] = 0.0;
    cpuWait[i] = 0.0;
    waitIO[i] = 0.0;
    waitBlockIO[i] = 0.0;
    waitSwap[i] = 0.0;
    waitPIO[i] = 0.0;
  }
}

static int
initMemory(void)
{
  ncpuconf = sysconf(_SC_NPROCESSORS_CONF);
  ncpuonln = sysconf(_SC_NPROCESSORS_ONLN);
#if defined (SOLARIS8)
  ncpumax = sysconf(_SC_NPROCESSORS_CONF);
#else
  ncpumax = sysconf(_SC_NPROCESSORS_MAX);
#endif
  CpuDevCnt = (int) ncpuonln;
  uint32_t i;

  cpuInfoArray = (CPUInfo *) realloc (cpuInfoArray, ncpuconf * sizeof (struct CPUInfo));
  if (0L == cpuInfoArray)
    return -1;
  cpuIndexArray = (CPUIndex *) realloc (cpuIndexArray, ncpuconf* sizeof (struct CPUIndex));
  if (0L == cpuIndexArray)
    return -1;
  cpuNewStatArray = (cpu_stat_t *) realloc (cpuNewStatArray, ncpuconf * sizeof (cpu_stat_t));
  if (0L == cpuNewStatArray)
    return -1;
  cpuOldStatArray = (cpu_stat_t *) realloc (cpuOldStatArray, ncpuconf * sizeof (cpu_stat_t));
  if (0L == cpuOldStatArray)
    return -1;

  cpuIdle = (double *) realloc (cpuIdle, ncpuconf * sizeof (double));
  cpuUser = (double *) realloc (cpuUser, ncpuconf * sizeof (double));
  cpuKernel = (double *) realloc (cpuKernel, ncpuconf * sizeof (double));
  cpuWait = (double *) realloc (cpuWait, ncpuconf * sizeof (double));
  waitIO = (double *) realloc (waitIO, ncpuconf * sizeof (double));
  waitBlockIO = (double *) realloc (waitBlockIO, ncpuconf * sizeof (double));
  waitSwap = (double *) realloc (waitSwap, ncpuconf * sizeof (double));
  waitPIO = (double *) realloc (waitPIO, ncpuconf * sizeof (double));

  setDefaultCPUValues();

  return 0;
}

static int
initCPUKStat(void)
{
  int32_t instance = 0;
  uint32_t i = 0;
  kstat_ctl_t* kc = 0L;
  kstat_t* ksp = 0L;
  kstat_named_t* kchipidp = 0L;

  kc = kstat_open();
  if (0L == kc)
    return -1;

  while (kstat_chain_update (kc) != 0);

  while (ncpuread < ncpuconf)
  {
    if (instance > (int) ncpumax)
      break;

    ksp = kstat_lookup(kc, "chip", instance++, 0L);
    if (ksp == (kstat_t *) 0L)
      continue;

    cpuInfoArray[ncpuread].instance = ksp->ks_instance;
    if (kstat_read (kc, ksp, 0L) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
    kchipidp = (kstat_named_t *) kstat_data_lookup (ksp, "chip_id");
    if (kchipidp)
    {
      cpuInfoArray[ncpuread].id = kchipidp->value.ui64;
      cpuIndexArray[ncpuread].id = (uint32_t) kchipidp->value.ui64;
      cpuIndexArray[ncpuread].index = ncpuread;
      ++ncpuread;
    }
  }
  if (0 == ncpuread)
  {
    (void) fprintf (stderr, "Error: Unable to retrieve any CPU Information!\n");
    kstat_close (kc);
    return -1;
  }

  while (kstat_chain_update(kc) != 0);

  for (i = 0; i < ncpuread; i++)
  {
    ksp = kstat_lookup(kc, "cpu_info", (int) cpuInfoArray[i].instance, 0L);
    if (0L == ksp)
    {
      perror("kstat_lookup"), strerror(errno);
      continue;
    }
    if (kstat_read (kc, ksp, 0L) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
    getCPUInfo (ksp, &(cpuInfoArray[i]));
  }

  while (kstat_chain_update(kc) != 0);

  for (i = 0; i < ncpuread; i++)
  {
    ksp = kstat_lookup(kc, "cpu_stat", (int) cpuInfoArray[i].instance, 0L);
    if (0L == ksp)
    {
      perror("kstat_lookup"), strerror(errno);
      continue;
    }
    if (kstat_read (kc, ksp, &(cpuNewStatArray[i])) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
  }
  kstat_close (kc);
  return 0;
}

static int
monitorCPUDelta(void)
{
  uint32_t i;
  kstat_ctl_t* kc = 0L;
  kstat_t* kmp = 0L;

  if (0UL == updateTime())
    return -1;

  kc = kstat_open();
  if (0L == kc)
    return -1;

  while (kstat_chain_update(kc) != 0);

  for (i = 0; i < ncpuread; i++)
  {
    kmp = kstat_lookup(kc, "cpu_stat", (int) cpuInfoArray[i].instance, 0L);
    if (0L == kmp)
    {
      perror("kstat_lookup"), strerror(errno);
      continue;
    }
    if (kstat_read (kc, kmp, &(cpuNewStatArray[i])) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }

    cpuIdle[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_IDLE],
                           cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_IDLE]);
    cpuUser[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_USER],
                           cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_USER]);
    cpuKernel[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_KERNEL],
                             cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_KERNEL]);
    cpuWait[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_WAIT],
                           cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_WAIT]);
    waitIO[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.wait[W_IO],
                          cpuOldStatArray[i].cpu_sysinfo.wait[W_IO]);
    waitBlockIO[i] = ddelta32 (cpuNewStatArray[i].cpu_syswait.iowait,
                               cpuOldStatArray[i].cpu_syswait.iowait);
    waitSwap[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.wait[W_SWAP],
                            cpuOldStatArray[i].cpu_sysinfo.wait[W_SWAP]);
    waitPIO[i] = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.wait[W_PIO],
                           cpuOldStatArray[i].cpu_sysinfo.wait[W_PIO]);

    cpuOldStatArray[i] = cpuNewStatArray[i];
  }
  kstat_close (kc);
  return 0;
}

void
initCpuInfo (struct SensorModul* sm)
{
  uint32_t i;
  char monitor[1024];

  CpuDevSM = sm;

  if (initMemory() != 0)
    return;

  if (initCPUKStat() != 0)
    return;

  updateCpuInfo();

  for (i = 0; i < ncpuread; i++)
  {
    (void) sprintf (monitor, "cpu%u/user", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUUser, printCPUUserInfo, CpuDevSM);

    (void) sprintf (monitor, "cpu%u/nice", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUNice, printCPUNiceInfo, CpuDevSM);

    (void) sprintf (monitor, "cpu%u/sys", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUSys, printCPUSysInfo, CpuDevSM);

    (void) sprintf (monitor, "cpu%u/idle", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUIdle, printCPUIdleInfo, CpuDevSM);

    (void) sprintf (monitor, "cpu%u/clock", cpuInfoArray[i].id);
    registerMonitor (monitor, "float", printCPUxClock, printCPUxClockInfo, CpuDevSM);
  }
}

void
exitCpuInfo(void)
{
  uint32_t i;
  char monitor[1024];

  for (i = 0; i < ncpuread; i++)
  {
    (void) sprintf (monitor, "cpu%u/user", cpuInfoArray[i].id);
    removeMonitor(monitor);
    (void) sprintf (monitor, "cpu%u/nice", cpuInfoArray[i].id);
    removeMonitor(monitor);
    (void) sprintf (monitor, "cpu%u/sys", cpuInfoArray[i].id);
    removeMonitor(monitor);
    (void) sprintf (monitor, "cpu%u/idle", cpuInfoArray[i].id);
    removeMonitor(monitor);
    (void) sprintf (monitor, "cpu%u/clock", cpuInfoArray[i].id);
    removeMonitor(monitor);
  }
}


int
updateCpuInfo(void)
{
  if (loop)
  {
    struct timeval tv;
    uint32_t i;

    (void) memset (&tv, '\0', sizeof (struct timeval));
    tv.tv_sec =1;

    for (i = 0; i < 3; i++)
    {
      monitorCPUDelta();
      select (0L, NULL, NULL, NULL, &tv);
    }
    loop = 0;
  }
  return (monitorCPUDelta());
}

void
printCPUUser (const char* cmd)
{
  int32_t cpuIndex;
  int32_t cpuId = getCpuId (cmd);

  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuUser[cpuIndex]);
}

void
printCPUUserInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);

  if (cpuId < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "CPU%ld User Load\t0\t100\t%%\n", cpuId);
}

void
printCPUNice (const char* cmd)
{
  int32_t cpuIndex;
  int32_t cpuId = getCpuId (cmd);

  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuWait[cpuIndex]);
}

void
printCPUNiceInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);

  if (cpuId < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "CPU%ld Nice Load\t0\t100\t%%\n", cpuId);
}

void
printCPUSys(const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuKernel[cpuIndex]);
}

void
printCPUSysInfo(const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);

  if (cpuId < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "CPU%ld Kernel Load\t0\t100\t%%\n", cpuId);
}

void
printCPUIdle(const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuIdle[cpuIndex]);
}

void
printCPUIdleInfo(const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);

  if (cpuId < 0)
    return;

  (void) fprintf (CurrentClient, "CPU%ld Idle Load\t0\t100\t%%\n", cpuId);
}

void printCPUxClock (const char* cmd)
{
  int32_t cpuIndex;
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", (double) cpuInfoArray[cpuIndex].clockMHz);
}

void printCPUxClockInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuInfo();
  (void) fprintf (CurrentClient, "CPU%ld Clock Frequency\t0\t0\tMHz\n", cpuId);
}

// vim: ts=2 sw=2 et

