/*
    KSysGuard, the KDE System Guard

    Copyright (c) 2000-2001 Chris Schlaeger <cs@kde.org>
    Solaris 8 9 and 10 support by Stefan Teleman <stefan.teleman@gmail.com>

    This program is free software; you can redistribute it and/or
    modify it under the terms of version 2 of the GNU General Public
    License as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "config.h"

#include <kstat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <inttypes.h>
#include <sys/types.h>
#include <sys/sysinfo.h>
#include <errno.h>

#include "Command.h"
#include "ccont.h"
#include "Common.h"
#include "CpuStat.h"
#include "ksysguardd.h"

typedef uint32_t CPUInstance;
typedef uint64_t CPUId;

typedef struct CPUStatInfo
{
  CPUId id;
  CPUInstance instance;
  uint32_t clockMHz;
  uint32_t online;
  char name[32];
  char cpuType[32];
  char fpuType[32];
  char implementation[256];
  char state[16];
  uint32_t anonpgin;
  uint32_t anonpgout;
  uint32_t anonfree;
  double idle;
  double kernel;
  double user;
  double wait;
  uint32_t pswitch;
  uint32_t execfree;
  uint32_t execpgin;
  uint32_t execpgout;
  uint32_t fspgin;
  uint32_t fspgout;
  uint32_t fsfree;
  uint32_t intr;
  uint32_t nthreads;
  uint32_t pgpgin;
  uint32_t pgpgout;
  uint32_t pgswapin;
  uint32_t pgswapout;
  uint32_t syscall;
  uint32_t sysexec;
  uint32_t sysfork;
  uint32_t sysread;
  uint32_t sysvfork;
  uint32_t syswrite;
  uint32_t trap;
  double waitio;
  double waitpio;
  double waitswap;
} CPUStatInfo;

typedef struct CPUIndex
{
  uint32_t id;
  uint32_t index;
} CPUIndex;

extern int errno;

static int Dirty = 0;
static int CpuDevCnt = 0;
static struct SensorModul* CpuDevSM;

static unsigned long interval = 0UL;
static time_t ot = (time_t) 0;
static time_t nt = (time_t) 0;

static int loop = 1;
static uint32_t ncpuread = 0;
static long ncpuonln = 0L; // sysconf(_SC_NPROCESSORS_ONLN);
static long ncpuconf = 0L; // sysconf(_SC_NPROCESSORS_CONF);
static long ncpumax = 0L; // sysconf(_SC_NPROCESSORS_MAX);
static uint64_t physpages = 0ULL;
static t_memsize swaptotal = (t_memsize) 0ULL;
static cpu_stat_t* cpuOldStatArray = 0L;
static cpu_stat_t* cpuNewStatArray = 0L;
static CPUStatInfo* cpuInfoArray = 0L;
static CPUIndex* cpuIndexArray = 0L;

static double
ddelta32 (uint32_t n, uint32_t o)
{
  return (floor ((double) ((n - o) / interval)));
}

static uint32_t
uidelta32 (uint32_t n, uint32_t o)
{
  return ((n - o) / interval);
}

static int
isOnline (const CPUStatInfo* cpuInfo)
{
  return ((cpuInfo) ? cpuInfo->online : 0);
}

static int32_t
getCpuIndex (int32_t id)
{
  CPUId cpuId;
  uint32_t i;

  if (0L == cpuIndexArray)
    return -1;

  cpuId = (CPUId) id;
  for (i = 0; i < ncpuread; i++)
  {
    if (cpuIndexArray[i].id == id)
      return cpuIndexArray[i].index;
  }
  return -1;
}

static void
copyCPUInfoData (CPUStatInfo* from, CPUStatInfo* to)
{
  (void) strcpy (to->name, from->name);
  (void) strcpy (to->cpuType, from->cpuType);
  (void) strcpy (to->fpuType, from->fpuType);
  (void) strcpy (to->implementation, from->implementation);
  (void) strcpy (to->state, from->state);
}

static void
fillCPUStatInfo (uint32_t i)
{
  cpuInfoArray[i].anonfree = uidelta32 (cpuNewStatArray[i].cpu_vminfo.anonfree,
                                       cpuOldStatArray[i].cpu_vminfo.anonfree);
  cpuInfoArray[i].anonpgin = uidelta32 (cpuNewStatArray[i].cpu_vminfo.anonpgin,
                                       cpuOldStatArray[i].cpu_vminfo.anonpgin);
  cpuInfoArray[i].anonpgout = uidelta32 (cpuNewStatArray[i].cpu_vminfo.anonpgout,
                                        cpuOldStatArray[i].cpu_vminfo.anonpgout);
  cpuInfoArray[i].idle = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_IDLE],
                                   cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_IDLE]);
  cpuInfoArray[i].kernel = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_KERNEL],
                                     cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_KERNEL]);
  cpuInfoArray[i].user = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_USER],
                                   cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_USER]);
  cpuInfoArray[i].wait = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.cpu[CPU_WAIT],
                                   cpuOldStatArray[i].cpu_sysinfo.cpu[CPU_WAIT]);
  cpuInfoArray[i].pswitch = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.pswitch,
                                      cpuOldStatArray[i].cpu_sysinfo.pswitch);
  cpuInfoArray[i].execfree = uidelta32 (cpuNewStatArray[i].cpu_vminfo.execfree,
                                       cpuOldStatArray[i].cpu_vminfo.execfree);
  cpuInfoArray[i].execpgin = uidelta32 (cpuNewStatArray[i].cpu_vminfo.execpgin,
                                       cpuOldStatArray[i].cpu_vminfo.execpgin);
  cpuInfoArray[i].execpgout = uidelta32 (cpuNewStatArray[i].cpu_vminfo.execpgout,
                                        cpuOldStatArray[i].cpu_vminfo.execpgout);
  cpuInfoArray[i].fspgin = uidelta32 (cpuNewStatArray[i].cpu_vminfo.fspgin,
                                     cpuOldStatArray[i].cpu_vminfo.fspgin);
  cpuInfoArray[i].fspgout = uidelta32 (cpuNewStatArray[i].cpu_vminfo.fspgout,
                                      cpuOldStatArray[i].cpu_vminfo.fspgout);
  cpuInfoArray[i].fsfree = uidelta32 (cpuNewStatArray[i].cpu_vminfo.fsfree,
                                     cpuOldStatArray[i].cpu_vminfo.fsfree);
  cpuInfoArray[i].intr = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.intr,
                                   cpuOldStatArray[i].cpu_sysinfo.intr);
  cpuInfoArray[i].nthreads = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.nthreads,
                                       cpuOldStatArray[i].cpu_sysinfo.nthreads);
  cpuInfoArray[i].pgpgin = uidelta32 (cpuNewStatArray[i].cpu_vminfo.pgpgin,
                                     cpuOldStatArray[i].cpu_vminfo.pgpgin);
  cpuInfoArray[i].pgpgout = uidelta32 (cpuNewStatArray[i].cpu_vminfo.pgpgout,
                                      cpuOldStatArray[i].cpu_vminfo.pgpgout);
  cpuInfoArray[i].pgswapin = uidelta32 (cpuNewStatArray[i].cpu_vminfo.pgswapin,
                                       cpuOldStatArray[i].cpu_vminfo.pgswapin);
  cpuInfoArray[i].pgswapout = uidelta32 (cpuNewStatArray[i].cpu_vminfo.pgswapout,
                                        cpuOldStatArray[i].cpu_vminfo.pgswapout);
  cpuInfoArray[i].syscall = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.syscall,
                                      cpuOldStatArray[i].cpu_sysinfo.syscall);
  cpuInfoArray[i].sysexec = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.sysexec,
                                      cpuOldStatArray[i].cpu_sysinfo.sysexec);
  cpuInfoArray[i].sysfork = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.sysfork,
                                      cpuOldStatArray[i].cpu_sysinfo.sysfork);
  cpuInfoArray[i].sysread = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.sysread,
                                      cpuOldStatArray[i].cpu_sysinfo.sysread);
  cpuInfoArray[i].sysvfork = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.sysvfork,
                                       cpuOldStatArray[i].cpu_sysinfo.sysvfork);
  cpuInfoArray[i].syswrite = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.syswrite,
                                       cpuOldStatArray[i].cpu_sysinfo.syswrite);
  cpuInfoArray[i].trap = uidelta32 (cpuNewStatArray[i].cpu_sysinfo.trap,
                                   cpuOldStatArray[i].cpu_sysinfo.trap);
  cpuInfoArray[i].waitio = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.wait[W_IO],
                                     cpuOldStatArray[i].cpu_sysinfo.wait[W_IO]);
  cpuInfoArray[i].waitpio = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.wait[W_PIO],
                                      cpuOldStatArray[i].cpu_sysinfo.wait[W_PIO]);
  cpuInfoArray[i].waitswap = ddelta32 (cpuNewStatArray[i].cpu_sysinfo.wait[W_SWAP],
                                       cpuOldStatArray[i].cpu_sysinfo.wait[W_SWAP]);
}

static int32_t
getCpuId (const char* cmd)
{
  char cpuId[32];
  uint32_t i = 0;

  if (0L == cmd)
    return -1;

  const char* slash = strchr (cmd, '/');
  if (0L == slash)
    return -1;

  const char* ptr = cmd;
  ptr += 3 * sizeof(char);
  (void) memset (cpuId, '\0', (size_t) 32);

  while (ptr != slash)
    cpuId[i++] = *ptr++;

  return (atoi(cpuId));
}

static unsigned long
updateTime (void)
{
  ot = nt;
  nt = time(0);

  if ((time_t) 0 == ot)
    return 0UL;

  interval = (unsigned long) nt - ot;
  return interval;
}

static void
getCpuInfo (kstat_t* kmp, CPUStatInfo* cpuInfo)
{
  kstat_named_t* kcpuinfop = 0L;

  if ((0 == kmp) || (0 == cpuInfo))
    return;

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "chip_id");
  if (kcpuinfop)
    cpuInfo->id = kcpuinfop->value.ui64;

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "clock_MHz");
  if (kcpuinfop)
    cpuInfo->clockMHz = kcpuinfop->value.ui32;

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "cpu_type");
  if (kcpuinfop)
    (void) strcpy (cpuInfo->cpuType, kcpuinfop->value.c);

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "fpu_type");
  if (kcpuinfop)
    (void) strcpy (cpuInfo->fpuType, kcpuinfop->value.c);

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "implementation");
  if (kcpuinfop)
  {
#if defined (SOLARIS8)
    (void) strcpy (cpuInfo->implementation, kcpuinfop->value.c);
#else
    (void) strcpy (cpuInfo->implementation, kcpuinfop->value.string.addr.ptr);
#endif
  }

  kcpuinfop = (kstat_named_t *) kstat_data_lookup(kmp, "state");
  if (kcpuinfop)
  {
    (void) strcpy (cpuInfo->state, kcpuinfop->value.c);
    cpuInfo->online = (0 == strcmp(kcpuinfop->value.c, "on-line"));
  }
}

static void
setDefaultCPUValues (void)
{
  uint32_t i;

  for (i = 0; i < (uint32_t) ncpuconf; ++i)
  {
    (void) memset ((void *) &(cpuInfoArray[i]), '\0', sizeof (struct CPUStatInfo));
    (void) memset ((void *) &(cpuIndexArray[i]), '\0', sizeof (struct CPUIndex));
    (void) memset ((void *) &(cpuNewStatArray[i]), '\0', sizeof (cpu_stat_t));
    (void) memset ((void *) &(cpuOldStatArray[i]), '\0', sizeof (cpu_stat_t));
  }
}

static int
initMemory(void)
{
  ncpuconf = sysconf(_SC_NPROCESSORS_CONF);
  ncpuonln = sysconf(_SC_NPROCESSORS_ONLN);
#if defined (SOLARIS8)
  ncpumax = sysconf(_SC_NPROCESSORS_CONF);
#else
  ncpumax = sysconf(_SC_NPROCESSORS_MAX);
#endif
  CpuDevCnt = (int) ncpuonln;
  uint32_t i;

  cpuInfoArray = (CPUStatInfo *) realloc (cpuInfoArray, ncpuconf * sizeof (struct CPUStatInfo));
  if (0L == cpuInfoArray)
    return -1;
  cpuIndexArray = (CPUIndex *) realloc (cpuIndexArray, ncpuconf* sizeof (struct CPUIndex));
  if (0L == cpuIndexArray)
    return -1;
  cpuNewStatArray = (cpu_stat_t *) realloc (cpuNewStatArray, ncpuconf * sizeof (cpu_stat_t));
  if (0L == cpuNewStatArray)
    return -1;
  cpuOldStatArray = (cpu_stat_t *) realloc (cpuOldStatArray, ncpuconf * sizeof (cpu_stat_t));
  if (0L == cpuOldStatArray)
    return -1;

  setDefaultCPUValues();
  return 0;
}

static int
initCPUKStat(void)
{
  int32_t instance = 0;
  uint32_t i = 0;
  kstat_ctl_t* kc = 0L;
  kstat_t* ksp = 0L;
  kstat_named_t* kchipidp = 0L;

  kc = kstat_open();
  if (0L == kc)
    return -1;

  while (kstat_chain_update (kc) != 0);

  while (ncpuread < ncpuconf)
  {
    if (instance > (int) ncpumax)
      break;

    ksp = kstat_lookup(kc, "chip", instance++, 0L);
    if (ksp == (kstat_t *) 0L)
      continue;

    cpuInfoArray[ncpuread].instance = ksp->ks_instance;
    if (kstat_read (kc, ksp, 0L) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
    kchipidp = (kstat_named_t *) kstat_data_lookup (ksp, "chip_id");
    if (kchipidp)
    {
      cpuInfoArray[ncpuread].id = kchipidp->value.ui64;
      cpuIndexArray[ncpuread].id = (uint32_t) kchipidp->value.ui64;
      cpuIndexArray[ncpuread].index = ncpuread;
      ++ncpuread;
    }
  }
  if (0 == ncpuread)
  {
    (void) fprintf (stderr, "Error: Unable to retrieve any CPU Information!\n");
    kstat_close (kc);
    return -1;
  }

  while (kstat_chain_update(kc) != 0);

  for (i = 0; i < ncpuread; i++)
  {
    ksp = kstat_lookup(kc, "cpu_info", (int) cpuInfoArray[i].instance, 0L);
    if (0L == ksp)
    {
      perror("kstat_lookup"), strerror(errno);
      continue;
    }
    if (kstat_read (kc, ksp, 0L) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
    getCpuInfo (ksp, &(cpuInfoArray[i]));
  }

  while (kstat_chain_update(kc) != 0);

  for (i = 0; i < ncpuread; i++)
  {
    ksp = kstat_lookup(kc, "cpu_stat", (int) cpuInfoArray[i].instance, 0L);
    if (0L == ksp)
    {
      perror("kstat_lookup"), strerror(errno);
      continue;
    }
    if (kstat_read (kc, ksp, &(cpuNewStatArray[i])) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
  }
  kstat_close (kc);
  return 0;
}

static int
monitorCPUDelta(void)
{
  uint32_t i;
  kstat_ctl_t* kc = 0L;
  kstat_t* kmp = 0L;

  if (0UL == updateTime())
    return -1;

  kc = kstat_open();
  if (0L == kc)
    return -1;

  while (kstat_chain_update(kc) != 0);

  for (i = 0; i < ncpuread; i++)
  {
    kmp = kstat_lookup(kc, "cpu_stat", (int) cpuInfoArray[i].instance, 0L);
    if (0L == kmp)
    {
      perror("kstat_lookup"), strerror(errno);
      continue;
    }
    if (kstat_read (kc, kmp, &(cpuNewStatArray[i])) == -1)
    {
      perror("kstat_read"), strerror(errno);
      continue;
    }
    fillCPUStatInfo (i);
    cpuOldStatArray[i] = cpuNewStatArray[i];
    copyCPUInfoData (&(cpuOldStatArray[i]), &(cpuNewStatArray[i]));
  }
  kstat_close (kc);
  return 0;
}

void
initCpuStatInfo (struct SensorModul* sm)
{
  uint32_t i;
  char monitor[1024];

  CpuDevSM = sm;

  if (initMemory() != 0)
    return;

  if (initCPUKStat() != 0)
    return;

  updateCpuStatInfo();

  for (i = 0; i < ncpuread; i++)
  {
    (void) sprintf (monitor, "cpu%u/anonfree", cpuInfoArray[i].id);
    registerMonitor(monitor, "integer", printCPUStatAnonFree, printCPUStatAnonFreeInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/anonpgin", cpuInfoArray[i].id);
    registerMonitor(monitor, "integer", printCPUStatAnonPGIn, printCPUStatAnonPGInInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/anonpgout", cpuInfoArray[i].id);
    registerMonitor(monitor, "integer", printCPUStatAnonPGOut, printCPUStatAnonPGOutInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/cpuidle", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUStatCpuIdle, printCPUStatCpuIdleInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/cpuuser", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUStatCpuUser, printCPUStatCpuUserInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/cpukernel", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUStatCpuKernel, printCPUStatCpuKernelInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/cpuwait", cpuInfoArray[i].id);
    registerMonitor(monitor, "float", printCPUStatCpuWait, printCPUStatCpuWaitInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/pswitch", cpuInfoArray[i].id);
    registerMonitor(monitor, "integer", printCPUStatPSwitch, printCPUStatPSwitchInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/execfree", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatExecFree, printCPUStatExecFreeInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/execpgin", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatExecPGIn, printCPUStatExecPGInInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/execpgout", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatExecPGOut, printCPUStatExecPGOutInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/fspgin", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatFsPGIn, printCPUStatFsPGInInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/fspgout", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatFsPGOut, printCPUStatFsPGOutInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/fsfree", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatFsFree, printCPUStatFsFreeInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/intr", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatIntr, printCPUStatIntrInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/nthreads", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatNThreads, printCPUStatNThreadsInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/pgpgin", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatPGIn, printCPUStatPGInInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/pgpgout", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatPGOut, printCPUStatPGOutInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/pgswapin", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSwapIn, printCPUStatSwapInInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/pgswapout", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSwapOut, printCPUStatSwapOutInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/syscall", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSysCall, printCPUStatSysCallInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/sysread", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSysRead, printCPUStatSysReadInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/syswrite", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSysWrite, printCPUStatSysWriteInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/sysfork", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSysFork, printCPUStatSysForkInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/sysvfork", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSysVFork, printCPUStatSysVForkInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/sysexec", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatSysExec, printCPUStatSysExecInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/trap", cpuInfoArray[i].id);
    registerMonitor (monitor, "integer", printCPUStatTrap, printCPUStatTrapInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/waitio", cpuInfoArray[i].id);
    registerMonitor (monitor, "float", printCPUStatWaitIO, printCPUStatWaitIOInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/waitpio", cpuInfoArray[i].id);
    registerMonitor (monitor, "float", printCPUStatWaitPIO, printCPUStatWaitPIOInfo, CpuDevSM);
    (void) sprintf (monitor, "cpu%u/waitswap", cpuInfoArray[i].id);
    registerMonitor (monitor, "float", printCPUStatWaitSwap, printCPUStatWaitSwapInfo, CpuDevSM);
  }
}

void
exitCpuStatInfo(void)
{
  uint32_t i;
  char monitor[1024];

  for (i = 0; i < ncpuread; i++)
  {
		(void) sprintf (monitor, "cpu%u/anonfree", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/anonpgin", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/anonpgout", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/cpuidle", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/cpuuser", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/cpukernel", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/cpuwait", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/pswitch", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/execfree", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/execpgin", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/execpgout", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/fspgin", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/fspgout", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/fsfree", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/intr", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/nthreads", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/pgpgin", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/pgpgout", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/pgswapin", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/pgswapout", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/syscall", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/sysread", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/syswrite", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/sysfork", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/sysvfork", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/sysexec", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/trap", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/waitio", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/waitpio", cpuInfoArray[i].id);
    removeMonitor (monitor);
		(void) sprintf (monitor, "cpu%u/waitswap", cpuInfoArray[i].id);
    removeMonitor (monitor);
  }
}


int
updateCpuStatInfo(void)
{
  int ret = 0;

  if (loop)
  {
    struct timeval tv;
    uint32_t i;

    (void) memset (&tv, '\0', sizeof (struct timeval));
    tv.tv_sec =1;

    for (i = 0; i < 3; i++)
    {
      ret = monitorCPUDelta();
      select (0L, NULL, NULL, NULL, &tv);
    }
    swaptotal = getSwapTotal();
    physpages = getPhysPages();
    loop = 0;
  }
  else
  {
    ret = monitorCPUDelta();
    swaptotal = getSwapTotal();
    physpages = getPhysPages();
  }
  return ret; 
}

void printCPUStatAnonFree (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].anonfree);
}

void printCPUStatAnonFreeInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Anon Pages Freed\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatAnonPGIn (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].anonpgin);
}

void printCPUStatAnonPGInInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Anon Pages Paged In\t0\t%llu\tPages\n", cpuId, physpages);

}

void printCPUStatAnonPGOut (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if ( cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].anonpgout);
}

void printCPUStatAnonPGOutInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Anon Pages Paged Out\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatCpuIdle (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].idle);
}

void printCPUStatCpuIdleInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Idle Time\t0\t100\t%%\n", cpuId);
}

void printCPUStatCpuKernel (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].kernel);
}

void printCPUStatCpuKernelInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Kernel Time\t0\t100\t%%\n", cpuId);
}

void printCPUStatCpuUser (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].user);
}

void printCPUStatCpuUserInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld User Time\t0\t100\t%%\n", cpuId);
}


void printCPUStatCpuWait (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].wait);
}


void printCPUStatCpuWaitInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Wait Time\t0\t100\t%%\n", cpuId);
}

void printCPUStatPSwitch (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].pswitch);
}

void printCPUStatPSwitchInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Context Switches\t0\t%u\tSwitches\n", cpuId, UINT32_MAX);
}

void printCPUStatExecFree (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].execfree);
}

void printCPUStatExecFreeInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Executable Pages Freed\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatExecPGIn (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].execpgin);
}

void printCPUStatExecPGInInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Executable Pages Paged In\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatExecPGOut (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].execpgout);
}

void printCPUStatExecPGOutInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Executable Pages Paged Out\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatFsPGIn (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].fspgin);
}

void printCPUStatFsPGInInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Filesystem Pages Paged In\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatFsPGOut (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].fspgout);
}

void printCPUStatFsPGOutInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Filesystem Pages Paged Out\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatFsFree (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].fsfree);
}

void printCPUStatFsFreeInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Filesystem Pages Free\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatIntr (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].intr);
}

void printCPUStatIntrInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Device Interrupts\t0\t%u\tInterrupts\n", cpuId, UINT32_MAX);
}

void printCPUStatNThreads (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].nthreads);
}

void printCPUStatNThreadsInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Threads Created\t0\t%u\tThreads\n", cpuId, UINT32_MAX);
}

void printCPUStatPGIn (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].pgpgin);
}

void printCPUStatPGInInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Pages Paged In\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatPGOut (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].pgpgout);
}

void printCPUStatPGOutInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Pages Paged Out\t0\t%llu\tPages\n", cpuId, physpages);
}

void printCPUStatSysCall (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].syscall);
}

void printCPUStatSysCallInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld System Calls\t0\t%u\tCalls\n", cpuId, UINT32_MAX);
}

void printCPUStatSysExec (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].sysexec);
}

void printCPUStatSysExecInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Exec System Calls\t0\t%u\tCalls\n", cpuId, UINT32_MAX);
}

void printCPUStatSysFork (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].sysfork);
}

void printCPUStatSysForkInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Fork System Calls\t0\t%u\tCalls\n", cpuId, UINT32_MAX);
}

void printCPUStatSysRead (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].sysread);
}

void printCPUStatSysReadInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Read System Calls\t0\t%u\tCalls\n", cpuId, UINT32_MAX);
}

void printCPUStatSysVFork (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].sysvfork);
}

void printCPUStatSysVForkInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld VFork System Calls\t0\t%u\tCalls\n", cpuId, UINT32_MAX);
}

void printCPUStatSysWrite (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].syswrite);
}

void printCPUStatSysWriteInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Write System Calls\t0\t%u\tCalls\n", cpuId, UINT32_MAX);
}

void printCPUStatTrap (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].trap);
}

void printCPUStatTrapInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Traps\t0\t%u\tTraps\n", cpuId, UINT32_MAX);
}

void printCPUStatWaitIO (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].waitio);
}

void printCPUStatWaitIOInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld IO Wait\t0\t100\t%%\n", cpuId);
}

void printCPUStatWaitPIO (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].waitpio);
}

void printCPUStatWaitPIOInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Page IO Wait\t0\t100\t%%\n", cpuId);
}

void printCPUStatWaitSwap (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%.2lf\n", cpuInfoArray[cpuIndex].waitswap);
}

void printCPUStatWaitSwapInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Swap Wait\t0\t100\t%%\n", cpuId);
}

void printCPUStatSwapIn (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].pgswapin);
}


void printCPUStatSwapInInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Pages Swapped In\t0\t%llu\tPages\n", cpuId, swaptotal);
}

void printCPUStatSwapOut (const char* cmd)
{
  int32_t cpuIndex;

  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  cpuIndex = getCpuIndex (cpuId);
  if (cpuIndex < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "%u\n", cpuInfoArray[cpuIndex].pgswapout);
}


void printCPUStatSwapOutInfo (const char* cmd)
{
  int32_t cpuId = getCpuId (cmd);
  if (cpuId < 0)
    return;

  updateCpuStatInfo();
  (void) fprintf (CurrentClient, "CPU%ld Pages Swapped Out\t0\t%llu\tPages\n", cpuId, swaptotal);
}


// vim: ts=2 sw=2 et

