/*
  KSysGuard, the KDE System Guard
  
  Copyright (c) 2005 Stefan Teleman <steleman@acm.org>
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of version 2 of the GNU General Public
  License as published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <config.h>

#include <stdio.h>
#include <sys/types.h>
#include <sys/dkio.h>
#include <sys/stat.h>
#include <sys/mkdev.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <inttypes.h>
#include <kstat.h>
#include <fcntl.h>
#include <sys/fcntl.h>
#include <sys/param.h>
#include <sys/mnttab.h>
#include <sys/mntent.h>
#include <sys/filio.h>
#include <sys/sysinfo.h>
#include <dirent.h>
#include <nfs/nfs.h>
#include <nfs/nfs_clnt.h>

#include "Command.h"
#include "ccont.h"
#include "NFSStat.h"
#include "ksysguardd.h"

typedef struct NFSMountInfo
{
  int instance;
  dev_t device;
  uint32_t major; // mnt_major
  uint32_t minor; // mnt_minor
  char nfsMountPoint[MAXPATHLEN]; // mnt_mountp
  char nfsMountFrom[MAXPATHLEN]; // mnt_special
  char nfsMountOptions[MNT_LINE_MAX]; // mnt_mntopts
  char kstatName[KSTAT_STRLEN];
  char kstatClass[KSTAT_STRLEN];
  struct mntinfo_kstat mik;
  kstat_io_t kstatIO;
} NFSMountInfo;

static void printKStat (const kstat_t* ksp);
static void printKIO (const kstat_io_t* kip);
static void printMntinfo (const struct mntinfo_kstat* mntp);
static void printNFSMountInfo (const NFSMountInfo* nmp);
static void printExtMnttab (const struct extmnttab* emtp);
static void addInstance (int instance);
static int findInstance (int instance);
static int updateInstances (void);
static int updateKStatList (void);
static void clearNFSMount (void);
static int updateNFSMount (void);
static void copyNFSMountInfo (NFSMountInfo* to, const struct extmnttab* const from);
static void copyMntinfoKStat (struct mntinfo_kstat* to, const struct mntinfo_kstat* const from);
static void copyKStatIO (kstat_io_t* to, const kstat_io_t* const from);
static void copyKStat (NFSMountInfo* to, const kstat_t* const from);
static const NFSMountInfo* const findNFSMountInfo (uint_t minor);
static char* getNFSMountPoint (const char* cmd);
static NFSMountInfo* addNFSMountInfoElement (size_t currentSize);

static const char* driver = "nfs";
static struct SensorModul* DiskStatSM = 0L;
static int numInstances = 0;
static int numNFSInfo = 0;
static int* instancePtr = 0L;
static NFSMountInfo* nfsInfoArray = 0L;
static char device[MAXPATHLEN];
static struct timeval tv;

extern int errno;

static char*
getNFSMountPoint (const char* cmd)
{
  char* ptr;
  char format[32];

  (void) memset (device, '\0', sizeof (device));
  (void) memset (format, '\0', 32);
  (void) sprintf (format, "nfs%%%us", (uint32_t) MAXPATHLEN);
  (void) sscanf (cmd, format, device);

  ptr = (char *) rindex (device, '/');
  if (ptr)
  {
    *ptr = '\0';
    return device;
  }
  return 0L;
}

void
printKStat (const kstat_t* ksp)
{
  if (ksp)
    {
      (void) fprintf (stderr, "ks_module = %s\n", ksp->ks_module);
      (void) fprintf (stderr, "\tks_name = %s\n", ksp->ks_name);
      (void) fprintf (stderr, "\tks_instance = %ld\n", ksp->ks_instance);
      (void) fprintf (stderr, "\tks_class = %s\n", ksp->ks_class);
      (void) fprintf (stderr, "\tks_kid = %ld\n", ksp->ks_kid);
      (void) fprintf (stderr, "\tks_type = %ld\n", ksp->ks_type);
    }
}

void
printKIO (const kstat_io_t* kip)
{
    if (kip)
    {
        (void) fprintf (stderr, "\t\tnread = %llu\n", kip->nread); // u_longlong_t 
        (void) fprintf (stderr, "\t\tnwritten = %llu\n", kip->nwritten); // u_longlong_t
        
        (void) fprintf (stderr, "\t\treads = %u\n", kip->reads); // uint_t
        (void) fprintf (stderr, "\t\twrites = %u\n", kip->writes); // uint_t
        
        (void) fprintf (stderr, "\t\twtime = %lld\n", kip->wtime ); // hrtime_t
        (void) fprintf (stderr, "\t\twlentime = %lld\n", kip->wlentime); // hrtime_t
        (void) fprintf (stderr, "\t\twlastupdate = %lld\n", kip->wlastupdate); // hrtime_t
        (void) fprintf (stderr, "\t\trtime = %lld\n", kip->rtime); // hrtime_t
        (void) fprintf (stderr, "\t\trlentime = %lld\n", kip->rlentime); // hrtime_t
        (void) fprintf (stderr, "\t\trlastupdate = %lld\n", kip->rlastupdate); // hrtime_t
        
        (void) fprintf (stderr, "\t\twcnt = %lld\n", kip->wcnt); // hrtime_t
        (void) fprintf (stderr, "\t\trcnt = %lld\n", kip->rcnt); // hrtime_t
    }
}

void
printMntinfo (const struct mntinfo_kstat* mp)
{
    uint32_t i;
    if (mp)
    {
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_proto = %s\n", mp->mik_proto);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_vers = %u\n", mp->mik_vers);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_flags = %u\n", mp->mik_flags);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_secmod = %u\n", mp->mik_secmod);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_curread = %u\n", mp->mik_curread);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_curwrite = %u\n", mp->mik_curwrite);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_timeo = %ld\n", mp->mik_timeo);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_retrans = %ld\n", mp->mik_retrans);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_acregmin = %u\n", mp->mik_acregmin);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_acregmax = %u\n", mp->mik_acregmax);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_acdirmin = %u\n", mp->mik_acdirmin);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_acdirmax = %u\n", mp->mik_acdirmax);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_noresponse = %u\n", mp->mik_noresponse);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_failover = %u\n", mp->mik_failover);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_remap = %u\n", mp->mik_remap);
        (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_curserver = %s\n", mp->mik_curserver);
        for (i = 0; i < NFS_CALLTYPES; ++i)
        {
            (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_timers[%i].srtt = %u\n", i, mp->mik_timers[i].srtt);
            (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_timers[%i].deviate = %u\n", i, mp->mik_timers[i].deviate);
            (void) fprintf (stderr, "\t\tmntinfo_kstat->mik_timers[%i].rtxcur = %u\n", i, mp->mik_timers[i].rtxcur);
        }
    }
}

void
addInstance (int instance)
{
    if (findInstance (instance) == -1)
    {
        instancePtr = (int *) realloc (instancePtr, (size_t) (numInstances * sizeof(int)) + sizeof(int));
        instancePtr[numInstances++] = instance;
    }
}

NFSMountInfo*
addNewNFSMountInfo (size_t currentSize)
{
    NFSMountInfo* ptr;
    NFSMountInfo* tmp;
    uint32_t i;

    ptr = (NFSMountInfo *) malloc ((1 + currentSize) * sizeof (NFSMountInfo));
    if (0L == ptr)
    {
        perror("malloc"), strerror(errno);
        return 0L;
    }
    if (0 == currentSize)
    {
        nfsInfoArray = ptr;
        return (&(nfsInfoArray[currentSize]));
    }
    
    for (i = 0; i < (uint32_t) currentSize; ++i)
    {
      (void) memset (&(ptr[i]), '\0', sizeof (NFSMountInfo));
      (void) memcpy (&(ptr[i]), &(nfsInfoArray[i]), sizeof (NFSMountInfo));
    }

    tmp = nfsInfoArray;
    nfsInfoArray = ptr;
    free (tmp);
    return (&(nfsInfoArray[currentSize]));
}


int
findInstance (int instance)
{
    int i;

    if (instancePtr == 0L)
        return -1;

    for (i = 0; i < numInstances; ++i)
    {
        if (instancePtr[i] == instance)
            return instancePtr[i];
    }
    return -1;
}

const NFSMountInfo* const
findNFSMountInfo (uint_t minor)
{
    uint32_t i;

    if ((0L == nfsInfoArray) || (0 == numNFSInfo))
        return 0L;

    for (i = 0; i < numNFSInfo; ++i)
        if (nfsInfoArray[i].minor == minor)
            return (&nfsInfoArray[i]);

    return 0L;
}


int
updateInstances (void)
{
    kstat_ctl_t* kc = 0L;
    kstat_t* ks = 0L;
    kstat_io_t kio;
    uint32_t i;
    
    if ((kc = kstat_open()) == (kstat_ctl_t *) NULL)
    {
        perror("kstat_open"), strerror(errno);
        return -1;
    }
    if ((ks = kstat_lookup (kc, "nfs", -1, NULL)) == (kstat_t *) NULL)
    {
        perror("kstat_lookup"), strerror(errno);
        kstat_close(kc);
        return -1;
    }

    do
    {
        while (kstat_chain_update(kc) != 0);
        if (strcmp (ks->ks_module, "nfs") == 0)
        {
            addInstance (ks->ks_instance);
            if (ks->ks_type == KSTAT_TYPE_IO)
            {
                if (kstat_read (kc, ks, &kio) == -1)
                {
                    perror("kstat_read"), strerror(errno);
                    ks = ks->ks_next;
                    continue;
                }
            }
        }
        ks = ks->ks_next;
    } while (ks);
    kstat_close (kc);
    return 0;
}


int
updateKStatList (void)
{
    kstat_ctl_t* kc = 0L;
    kstat_t* ks = 0L;
    kstat_t* km = 0L;
    kstat_io_t kio;
    mntinfo_t mntinfo;
    struct mntinfo_kstat* mikPtr = 0L;
    const NFSMountInfo* nfsPtr = 0L;
    uint32_t i;
    
    if ((kc = kstat_open()) == (kstat_ctl_t *) NULL)
    {
        perror("kstat_open"), strerror(errno);
        return -1;
    }
    for (i = 0; i < numInstances; ++i)
    {
        if ((ks = kstat_lookup (kc, "nfs", instancePtr[i], NULL)) == (kstat_t *) NULL)
        {
            perror("kstat_lookup"), strerror(errno);
            continue;
        }
        
        while (kstat_chain_update(kc) != 0);
        
        while (ks)
        {
            if (strcmp (ks->ks_module, "nfs") != 0)
            {
                ks = ks->ks_next;
                continue;
            }

            while (kstat_chain_update(kc) != 0);

            if ((ks->ks_type == KSTAT_TYPE_RAW) && (strcmp (ks->ks_class, "misc") == 0))
            {
                if (kstat_read (kc, ks, 0L) == -1)
                {
                    perror("kstat_read"), strerror(errno);
                    continue;
                }
                mikPtr = (struct mntinfo_kstat *)(ks->ks_data);
                nfsPtr = findNFSMountInfo ((uint_t) ks->ks_instance);
                copyKStat (nfsPtr, ks);
                copyMntinfoKStat (&(nfsPtr->mik), mikPtr);
            }
            if ((ks->ks_type == KSTAT_TYPE_IO) && (strcmp (ks->ks_class, "nfs") == 0))
            {
                (void) memset (&kio, '\0', sizeof (kstat_io_t));
                if (kstat_read (kc, ks, &kio) == -1)
                    perror("kstat_read"), strerror(errno);
                else
                {
                  nfsPtr = findNFSMountInfo ((uint_t) ks->ks_instance);
                  copyKStat (nfsPtr, ks);
                  copyKStatIO (&(nfsPtr->kstatIO), &kio);
                }
            }
            ks = ks->ks_next;
        }
    }
    kstat_close(kc);
    return 0;
}

void
clearNFSMount (void)
{
    if (nfsInfoArray)
    {
        free (nfsInfoArray);
        nfsInfoArray = 0L;
        numNFSInfo = 0;
    }
}

int
updateNFSMount (void)
{
    FILE* fp;
    int ret;
    struct extmnttab mp;

    clearNFSMount();

    (void) memset (&mp, '\0', sizeof(struct extmnttab));
    if ((fp = fopen(MNTTAB, "r")) == (FILE *) NULL)
    {
        perror("fopen"), strerror(errno);
        return -1;
    }
    
    while (0 == getextmntent (fp, &mp, MNT_LINE_MAX))
    {
        if (0 == strcmp (mp.mnt_fstype, "nfs"))
        {
            if (0L == addNewNFSMountInfo (numNFSInfo))
            {
                clearNFSMount();
                rewind(fp);
                resetmnttab(fp);
                fclose(fp);
                return -1;
            }
            copyNFSMountInfo (&nfsInfoArray[numNFSInfo], &mp);
            ++numNFSInfo;
        }
    }
    rewind(fp);
    resetmnttab(fp);
    fclose(fp);
    return 0;
}

void
copyNFSMountInfo (NFSMountInfo* to, const struct extmnttab* const from)
{
    if (to && from)
    {
        (void) strcpy (to->nfsMountFrom, from->mnt_special);
        (void) strcpy (to->nfsMountPoint, from->mnt_mountp);
        (void) strcpy (to->nfsMountOptions, from->mnt_mntopts);
        to->major = from->mnt_major;
        to->minor = from->mnt_minor;
        to->device = makedev ((major_t) to->major, (minor_t) to->minor);
    }
}

void
copyMntinfoKStat (struct mntinfo_kstat* to, const struct mntinfo_kstat* const from)
{
  if ((0L == to) || (0L == from))
    return;

  (void) memcpy (to, from, sizeof (struct mntinfo_kstat));
}

void
copyKStatIO (kstat_io_t* to, const kstat_io_t* const from)
{
  if ((0L == to) || (0L == from))
    return;

  (void) memcpy (to, from, sizeof (kstat_io_t));
}

void
copyKStat (NFSMountInfo* to, const kstat_t* const from)
{
  if ((0L == to) || (0L == from))
    return;

  to->instance = from->ks_instance;
  (void) strcpy (to->kstatName, from->ks_name);
  (void) strcpy (to->kstatClass, from->ks_class);
}


void
printNFSMountInfo (const NFSMountInfo* nmp)
{
    if (nmp)
    {
        (void) fprintf (stderr, "\t\tnfsMountFrom: %s\n", nmp->nfsMountFrom);
        (void) fprintf (stderr, "\t\tnfsMountPoint: %s\n", nmp->nfsMountPoint);
        (void) fprintf (stderr, "\t\tnfsMountOptions: %s\n", nmp->nfsMountOptions);
        (void) fprintf (stderr, "\t\tdevice: 0x%x\n", nmp->device);
        (void) fprintf (stderr, "\t\tmajor: %u\n", nmp->major);
        (void) fprintf (stderr, "\t\tminor: %u\n", nmp->minor);
        printMntinfo (&(nmp->mik));
        printKIO (&(nmp->kstatIO));
    }
}

/*
 * public interface
 */

int
updateNFSStat (void)
{
    if (getNFSStatInfo() > 0)
        return 0;
    
    return -1;
}


int
getNFSStatInfo (void)
{
    if (updateNFSMount() != 0)
        return -1;
    if (updateInstances() != 0)
        return -1;
    if (updateKStatList() != 0)
        return -1;
    
    return 0;
}


void
initNFSStat (struct SensorModul* sm)
{
  char monitor[MAXPATHLEN + (16 * sizeof(char))];
  uint32_t i;

  DiskStatSM = sm;
  updateNFSStat();
  
  registerMonitor("nfs/list", "listview", printNFSStat, printNFSStatInfo, DiskStatSM);
  
  for (i = 0; i < numNFSInfo; ++i)
  {
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/bread", nfsInfoArray[i].nfsMountPoint);
      registerMonitor (monitor, "integer", printNFSStatBytesRead, printNFSStatBytesReadInfo, DiskStatSM);
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/bwritten", nfsInfoArray[i].nfsMountPoint);
      registerMonitor (monitor, "integer", printNFSStatBytesWritten, printNFSStatBytesWrittenInfo, DiskStatSM);
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/nreads", nfsInfoArray[i].nfsMountPoint);
      registerMonitor (monitor, "integer", printNFSStatNumReadOps, printNFSStatNumReadOpsInfo, DiskStatSM);
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/nwrites", nfsInfoArray[i].nfsMountPoint);
      registerMonitor (monitor, "integer", printNFSStatNumWriteOps, printNFSStatNumWriteOpsInfo, DiskStatSM);
  }
}

void
exitNFSStat (void)
{
  char monitor[MAXPATHLEN + (16 * sizeof(char))];
  uint32_t i;

  removeMonitor("nfs/list");

  for (i = 0; i < numNFSInfo; ++i)
  {
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/bread", nfsInfoArray[i].nfsMountPoint);
      removeMonitor (monitor);
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/bwritten", nfsInfoArray[i].nfsMountPoint);
      removeMonitor (monitor);
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/nreads", nfsInfoArray[i].nfsMountPoint);
      removeMonitor (monitor);
      (void) snprintf (monitor, sizeof(monitor), "nfs%s/nwrites", nfsInfoArray[i].nfsMountPoint);
      removeMonitor (monitor);
      free (nfsInfoArray);
      nfsInfoArray = 0L;
  }
}

void
checkNFSStat (void)
{
  struct extmnttab mp;
  unsigned int ctr = 0;
  int ret = 0;
  FILE* fp = 0L;
  uint32_t i;
  char line[1024];

  if ((fp = fopen (MNTTAB, "r")) == (FILE *) 0L)
  {
      perror("fopen"), strerror(errno);
      return;
  }
  (void) memset ((void *) &mp, '\0', sizeof (struct extmnttab));
  
  while (0 == getextmntent (fp, &mp, MNT_LINE_MAX))
  {
      if (0 == strcmp (mp.mnt_fstype, "nfs"))
          ++ctr;
  }
  resetmnttab (fp);
  fclose (fp);
  
  if (ctr != numNFSInfo)
  {
      exitNFSStat();
      initNFSStat (DiskStatSM);
  }
}

void
printNFSStat (const char* cmd)
{
  uint32_t i;
  
  for (i = 0; i < numNFSInfo; i++)
  {
      (void) fprintf (CurrentClient, "0x%x\t%u\t%u\t%s\t%s\t%s\t%s\t%s\t%u\t%u\t%u\t%u\t%u\t%i\t%i\t%s\t%u\t%llu\t%u\t%llu\n",
                      nfsInfoArray[i].device,
                      nfsInfoArray[i].major,
                      nfsInfoArray[i].minor,
                      nfsInfoArray[i].nfsMountPoint,
                      nfsInfoArray[i].nfsMountFrom,
                      nfsInfoArray[i].kstatName,
                      nfsInfoArray[i].kstatClass,
                      nfsInfoArray[i].mik.mik_proto,
                      nfsInfoArray[i].mik.mik_vers,
                      nfsInfoArray[i].mik.mik_flags,
                      nfsInfoArray[i].mik.mik_secmod,
                      nfsInfoArray[i].mik.mik_curread,
                      nfsInfoArray[i].mik.mik_curwrite,
                      nfsInfoArray[i].mik.mik_timeo,
                      nfsInfoArray[i].mik.mik_retrans,
                      nfsInfoArray[i].nfsMountOptions,
                      nfsInfoArray[i].kstatIO.reads,
                      nfsInfoArray[i].kstatIO.nread,
                      nfsInfoArray[i].kstatIO.writes,
                      nfsInfoArray[i].kstatIO.nwritten);
  }
  (void) fprintf (CurrentClient, "\n");
}


void
printNFSStatInfo (const char* cmd)
{
    (void) fprintf(CurrentClient, "Device\tMajor\tMinor\tNFSMountPoint\tNFSMountFrom\tName\tClass\tProtocol\tVersion\tFlags\tSecurityMode\tCurrentRead\tCurrentWrite\tTimeouts\tRetransmissions\tNFSMountOptions\tReadOperations\tBytesRead\tWriteOperations\tBytesWritten\ns\tD\tD\ts\ts\ts\ts\ts\tD\tD\tD\tD\tD\tD\tD\ts\tD\tD\tD\tD\n");
}


void
printNFSStatNumReadOps (const char* cmd)
{
    uint32_t i;
    char* nfsMountPoint = getNFSMountPoint (cmd);

    if (nfsMountPoint)
    {
        for (i = 0; i < numNFSInfo; i++)
        {
            if (0 == strcmp (nfsMountPoint, nfsInfoArray[i].nfsMountPoint))
                (void) fprintf (CurrentClient, "%u\n", nfsInfoArray[i].kstatIO.reads);
        }
    }
    (void) fprintf (CurrentClient, "\n");
}


void
printNFSStatNumReadOpsInfo (const char* cmd)
{
    (void) fprintf (CurrentClient, "Read Operations\t0\t-\tReads\n");
}

void
printNFSStatNumWriteOps (const char* cmd)
{
  uint32_t i;
  char* nfsMountPoint = getNFSMountPoint (cmd);

  if (nfsMountPoint)
  {
      for (i = 0; i < numNFSInfo; i++)
      {
          if (0 == strcmp (nfsMountPoint, nfsInfoArray[i].nfsMountPoint))
              (void) fprintf (CurrentClient, "%u\n", nfsInfoArray[i].kstatIO.writes);
      }
  }
  (void) fprintf (CurrentClient, "\n");
}


void
printNFSStatNumWriteOpsInfo (const char* cmd)
{
  (void) fprintf (CurrentClient, "Write Operations\t0\t-\tWrites\n");
}


void
printNFSStatBytesRead (const char* cmd)
{
    uint32_t i;
    char* nfsMountPoint = getNFSMountPoint (cmd);

    if (nfsMountPoint)
    {
        for (i = 0; i < numNFSInfo; i++)
        {
            if (0 == strcmp (nfsMountPoint, nfsInfoArray[i].nfsMountPoint))
                (void) fprintf (CurrentClient, "%llu\n", nfsInfoArray[i].kstatIO.nread);
        }
    }
}


void
printNFSStatBytesReadInfo (const char* cmd)
{
    (void) fprintf (CurrentClient, "Bytes Read\t0\t-\tBytes\n");
}


void
printNFSStatBytesWritten (const char* cmd)
{
    uint32_t i;
    char* nfsMountPoint = getNFSMountPoint (cmd);
    
    if (nfsMountPoint)
    {
        for (i = 0; i < numNFSInfo; i++)
        {
            if (0 == strcmp (nfsMountPoint, nfsInfoArray[i].nfsMountPoint))
                (void) fprintf (CurrentClient, "%llu\n", nfsInfoArray[i].kstatIO.nwritten);
        }
    }
}

void
printNFSStatBytesWrittenInfo (const char* cmd)
{
    (void) fprintf (CurrentClient, "Bytes Written\t0\t-\tBytes\n");
}

// vim: ts=2 sw=2 et



